// This software is entered into the public domain.  Any copies of
// this software or software derived from this software, in part or
// whole, should include this header text.  Distribution of this software
// is restricted to non-profit purposes only.
//
// <Standard disclaimer> - This application is intended for recreational
// use only.  The author is not responsible for damage of any type
// resulting from it's use.
//
// To compile:cc -ObjC WierdTiff.m -o WierdTiff -g -lm -lMedia_s -lNeXT_s
//
// Author: Robert Kieffer
//
// WierdTiff.m...
// An application for screwing with the transparent regions of an image.
// This app will replace regions of alpha = 0 with the color specified by
// the red, green, blue, and alpha values supplied. These values should
// be between 0 and 255.  The most interesting results occur when red,
// green, or blue is greater than alpha.
//
// Be forewarned, the input image must be a 24-bit, alpha, RGB, meshed-format,
// TIFF.  (a standard 24-bit TIFF with alpha).

#import <appkit/appkit.h>

main(int argc, char *argv[])
	{
	NXStream *myStream;
	NXBitmapImageRep *myRep;
	u_char r,g,b,a;
	u_char *rd, *gd, *bd, *ad;
	int width, height, x, y;

	if (argc != 7)
		{
		printf("Usage: %s infile outfile red green blue alpha\n", argv[0]);
		exit(1);
		}

	// Get connection to windowserver
	[Application new];

	// Load the tiff file into a representation and get it's size
	myRep = [[NXBitmapImageRep alloc] initFromFile:argv[1]];
	if (!myRep ||
		[myRep bitsPerSample] != 8 ||
		[myRep samplesPerPixel] != 4 ||
		[myRep colorSpace] != NX_RGBColorSpace ||
		[myRep isPlanar])
		{
		printf("Input image not is not in rgb, 24-bit, alpha format!\n");
		exit(1);
		}

	width = [myRep pixelsWide];
	height = [myRep pixelsHigh];
	rd = [myRep data];
	gd = rd+1;
	bd = gd+1;
	ad = bd+1;
	r = (u_char)atoi(argv[3]);
	g = (u_char)atoi(argv[4]);
	b = (u_char)atoi(argv[5]);
	a = (u_char)atoi(argv[6]);

	for (y = 0; y < height; y++)
		for (x = 0; x < width; x++, rd+=4, gd+=4, bd+=4, ad+=4)
			{
			if (!*ad)
				{
				*rd = r;
				*gd = g;
				*bd = b;
				*ad = a;
				}
			}

	// Write to disk
	myStream = NXOpenMemory(NULL, 0, NX_WRITEONLY);
	if (myStream)
		{
		[myRep writeTIFF:myStream];
		NXSaveToFile(myStream, argv[2]);
		NXClose(myStream);
		}
	}
